#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT 0x0400
#include <windows.h>
#include <wincrypt.h>
#include <richedit.h>
#include "ConvFunc.h"
#include "AkelDLL.h"
#include "Resources\Resource.h"


//Include conversion functions
#define dec2hexW
#define hex2decW
#define str2hexW
#define hex2strW
#include "ConvFunc.h"

//Defines
#define STRID_OK             1
#define STRID_CANCEL         2
#define STRID_ENCRYPTION     3
#define STRID_DECRYPTION     4
#define STRID_PASS1          5
#define STRID_PASS2          6
#define STRID_PASSDONTMATCH  7
#define STRID_NOSELECTION    8

#define ENCRYPT_ALGORITHM  CALG_RC4

//Functions prototypes
BOOL CALLBACK CryptDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK CryptDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
BOOL SetupCryptoClient();
BOOL EncryptString(unsigned char *lpData, DWORD *dwDataLen, unsigned char *lpKey, DWORD dwKeyLen);
BOOL DecryptString(unsigned char *lpData, DWORD *dwDataLen, unsigned char *lpKey, DWORD dwKeyLen);
char* GetLangStringA(LANGID wLangID, int nStringID);
wchar_t* GetLangStringW(LANGID wLangID, int nStringID);
void InitCommon(PLUGINDATA *pd);

//Global variables
HINSTANCE hInstanceDLL;
HWND hMainWnd;
HWND hWndEdit;
HICON hMainIcon;
BOOL bOldWindows;
BOOL bAkelEdit;
LANGID wLangSystem;
BOOL bInitCommon=FALSE;
BOOL bEncrypt;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="Crypt";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Encrypt(PLUGINDATA *pd)
{
  CHARRANGE cr;

  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=FALSE;
    return;
  }

  InitCommon(pd);
  bEncrypt=TRUE;
  SendMessage(hWndEdit, EM_EXGETSEL, 0, (LPARAM)&cr);

  if (cr.cpMin != cr.cpMax)
  {
    if (bOldWindows)
      DialogBoxA(hInstanceDLL, MAKEINTRESOURCEA(IDD_ENCRYPT), hMainWnd, (DLGPROC)CryptDlgProcA);
    else
      DialogBoxW(hInstanceDLL, MAKEINTRESOURCEW(IDD_ENCRYPT), hMainWnd, (DLGPROC)CryptDlgProcW);
  }
  else
  {
    if (bOldWindows)
      MessageBoxA(hMainWnd, GetLangStringA(wLangSystem, STRID_NOSELECTION), "Crypt plugin", MB_OK|MB_ICONEXCLAMATION);
    else
      MessageBoxW(hMainWnd, GetLangStringW(wLangSystem, STRID_NOSELECTION), L"Crypt plugin", MB_OK|MB_ICONEXCLAMATION);
  }
}

extern "C" void __declspec(dllexport) Decrypt(PLUGINDATA *pd)
{
  CHARRANGE cr;

  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=FALSE;
    return;
  }

  InitCommon(pd);
  bEncrypt=FALSE;
  SendMessage(hWndEdit, EM_EXGETSEL, 0, (LPARAM)&cr);

  if (cr.cpMin != cr.cpMax)
  {
    if (bOldWindows)
      DialogBoxA(hInstanceDLL, MAKEINTRESOURCEA(IDD_DECRYPT), hMainWnd, (DLGPROC)CryptDlgProcA);
    else
      DialogBoxW(hInstanceDLL, MAKEINTRESOURCEW(IDD_DECRYPT), hMainWnd, (DLGPROC)CryptDlgProcW);
  }
  else
  {
    if (bOldWindows)
      MessageBoxA(hMainWnd, GetLangStringA(wLangSystem, STRID_NOSELECTION), "Crypt plugin", MB_OK|MB_ICONEXCLAMATION);
    else
      MessageBoxW(hMainWnd, GetLangStringW(wLangSystem, STRID_NOSELECTION), L"Crypt plugin", MB_OK|MB_ICONEXCLAMATION);
  }
}

BOOL CALLBACK CryptDlgProcA(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndPass1;
  static HWND hWndPass2;
  static HWND hWndOK;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndPass1=GetDlgItem(hDlg, IDC_PASS1);
    if (bEncrypt) hWndPass2=GetDlgItem(hDlg, IDC_PASS2);
    hWndOK=GetDlgItem(hDlg, IDOK);

    if (bEncrypt)
      SetDlgItemTextA(hDlg, IDC_GROUP_LABEL, GetLangStringA(wLangSystem, STRID_ENCRYPTION));
    else
      SetDlgItemTextA(hDlg, IDC_GROUP_LABEL, GetLangStringA(wLangSystem, STRID_DECRYPTION));
    SetDlgItemTextA(hDlg, IDC_PASS1_LABEL, GetLangStringA(wLangSystem, STRID_PASS1));
    if (bEncrypt) SetDlgItemTextA(hDlg, IDC_PASS2_LABEL, GetLangStringA(wLangSystem, STRID_PASS2));
    SetDlgItemTextA(hDlg, IDOK, GetLangStringA(wLangSystem, STRID_OK));
    SetDlgItemTextA(hDlg, IDCANCEL, GetLangStringA(wLangSystem, STRID_CANCEL));

    SendMessage(hWndPass1, EM_LIMITTEXT, (WPARAM)MAX_PATH, 0);
    if (bEncrypt) SendMessage(hWndPass2, EM_LIMITTEXT, (WPARAM)MAX_PATH, 0);
    EnableWindow(hWndOK, FALSE);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_PASS1)
    {
      if (GetWindowTextLengthA(hWndPass1))
        EnableWindow(hWndOK, TRUE);
      else
        EnableWindow(hWndOK, FALSE);
    }
    else if (LOWORD(wParam) == IDOK)
    {
      CHARRANGE cr;
      char szPass1[MAX_PATH];
      char szPass2[MAX_PATH];
      wchar_t *wszSelection;
      wchar_t *wszString;
      unsigned char *lpData;
      DWORD dwSelectionLen=0;
      DWORD dwDataLen=0;

      GetWindowTextA(hWndPass1, szPass1, MAX_PATH);
      if (bEncrypt) GetWindowTextA(hWndPass2, szPass2, MAX_PATH);

      if (bEncrypt)
      {
        if (lstrcmpA(szPass1, szPass2))
        {
          SetWindowTextA(hWndPass1, "");
          SetWindowTextA(hWndPass2, "");
          MessageBoxA(hDlg, GetLangStringA(wLangSystem, STRID_PASSDONTMATCH), "Crypt plugin", MB_OK|MB_ICONEXCLAMATION);
          return FALSE;
        }
      }

      if (wszSelection=(wchar_t *)SendMessage(hMainWnd, AKD_GETSELTEXTW, (WPARAM)hWndEdit, (LPARAM)&dwSelectionLen))
      {
        if (!bAkelEdit)
        {
          SendMessage(hWndEdit, EM_EXGETSEL, 0, (LPARAM)&cr);
          dwSelectionLen=cr.cpMax - cr.cpMin;
        }

        if (SetupCryptoClient())
        {
          if (bEncrypt)
          {
            lpData=(unsigned char *)wszSelection;
            dwDataLen=dwSelectionLen * sizeof(wchar_t);

            if (EncryptString(lpData, &dwDataLen, (unsigned char *)szPass1, lstrlenA(szPass1)))
            {
              if (wszString=(wchar_t *)GlobalAlloc(GPTR, (dwDataLen * 2 + 1) * sizeof(wchar_t)))
              {
                str2hexW(lpData, dwDataLen, wszString, dwDataLen * 2 + 1, TRUE);
                SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndEdit, (LPARAM)wszString);
                GlobalFree((HGLOBAL)wszString);
              }
            }
          }
          else
          {
            if (lpData=(unsigned char *)GlobalAlloc(GPTR, dwSelectionLen + 2))
            {
              dwDataLen=hex2strW(wszSelection, lpData, dwSelectionLen);

              if (DecryptString(lpData, &dwDataLen, (unsigned char *)szPass1, lstrlenA(szPass1)))
              {
                wszString=(wchar_t *)lpData;
                wszString[dwDataLen]='\0';

                SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndEdit, (LPARAM)wszString);
              }
              GlobalFree((HGLOBAL)lpData);
            }
          }
        }
        SendMessage(hMainWnd, AKD_FREETEXT, 0, (LPARAM)wszSelection);
      }
      EndDialog(hDlg, 0);
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

BOOL CALLBACK CryptDlgProcW(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hWndPass1;
  static HWND hWndPass2;
  static HWND hWndOK;

  if (uMsg == WM_INITDIALOG)
  {
    SendMessage(hDlg, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)hMainIcon);
    hWndPass1=GetDlgItem(hDlg, IDC_PASS1);
    if (bEncrypt) hWndPass2=GetDlgItem(hDlg, IDC_PASS2);
    hWndOK=GetDlgItem(hDlg, IDOK);

    if (bEncrypt)
      SetDlgItemTextW(hDlg, IDC_GROUP_LABEL, GetLangStringW(wLangSystem, STRID_ENCRYPTION));
    else
      SetDlgItemTextW(hDlg, IDC_GROUP_LABEL, GetLangStringW(wLangSystem, STRID_DECRYPTION));
    SetDlgItemTextW(hDlg, IDC_PASS1_LABEL, GetLangStringW(wLangSystem, STRID_PASS1));
    if (bEncrypt) SetDlgItemTextW(hDlg, IDC_PASS2_LABEL, GetLangStringW(wLangSystem, STRID_PASS2));
    SetDlgItemTextW(hDlg, IDOK, GetLangStringW(wLangSystem, STRID_OK));
    SetDlgItemTextW(hDlg, IDCANCEL, GetLangStringW(wLangSystem, STRID_CANCEL));

    SendMessage(hWndPass1, EM_LIMITTEXT, (WPARAM)MAX_PATH, 0);
    if (bEncrypt) SendMessage(hWndPass2, EM_LIMITTEXT, (WPARAM)MAX_PATH, 0);
    EnableWindow(hWndOK, FALSE);
  }
  else if (uMsg == WM_COMMAND)
  {
    if (LOWORD(wParam) == IDC_PASS1)
    {
      if (GetWindowTextLengthW(hWndPass1))
        EnableWindow(hWndOK, TRUE);
      else
        EnableWindow(hWndOK, FALSE);
    }
    else if (LOWORD(wParam) == IDOK)
    {
      CHARRANGE cr;
      char szPass1[MAX_PATH];
      char szPass2[MAX_PATH];
      wchar_t *wszSelection;
      wchar_t *wszString;
      unsigned char *lpData;
      DWORD dwSelectionLen=0;
      DWORD dwDataLen=0;

      GetWindowTextA(hWndPass1, szPass1, MAX_PATH);
      if (bEncrypt) GetWindowTextA(hWndPass2, szPass2, MAX_PATH);

      if (bEncrypt)
      {
        if (lstrcmpA(szPass1, szPass2))
        {
          SetWindowTextW(hWndPass1, L"");
          SetWindowTextW(hWndPass2, L"");
          MessageBoxW(hDlg, GetLangStringW(wLangSystem, STRID_PASSDONTMATCH), L"Crypt plugin", MB_OK|MB_ICONEXCLAMATION);
          return FALSE;
        }
      }

      if (wszSelection=(wchar_t *)SendMessage(hMainWnd, AKD_GETSELTEXTW, (WPARAM)hWndEdit, (LPARAM)&dwSelectionLen))
      {
        if (!bAkelEdit)
        {
          SendMessage(hWndEdit, EM_EXGETSEL, 0, (LPARAM)&cr);
          dwSelectionLen=cr.cpMax - cr.cpMin;
        }

        if (SetupCryptoClient())
        {
          if (bEncrypt)
          {
            lpData=(unsigned char *)wszSelection;
            dwDataLen=dwSelectionLen * sizeof(wchar_t);

            if (EncryptString(lpData, &dwDataLen, (unsigned char *)szPass1, lstrlenA(szPass1)))
            {
              if (wszString=(wchar_t *)GlobalAlloc(GPTR, (dwDataLen * 2 + 1) * sizeof(wchar_t)))
              {
                str2hexW(lpData, dwDataLen, wszString, dwDataLen * 2 + 1, TRUE);
                SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndEdit, (LPARAM)wszString);
                GlobalFree((HGLOBAL)wszString);
              }
            }
          }
          else
          {
            if (lpData=(unsigned char *)GlobalAlloc(GPTR, dwSelectionLen + 2))
            {
              dwDataLen=hex2strW(wszSelection, lpData, dwSelectionLen);

              if (DecryptString(lpData, &dwDataLen, (unsigned char *)szPass1, lstrlenA(szPass1)))
              {
                wszString=(wchar_t *)lpData;
                wszString[dwDataLen]='\0';

                SendMessage(hMainWnd, AKD_REPLACESELW, (WPARAM)hWndEdit, (LPARAM)wszString);
              }
              GlobalFree((HGLOBAL)lpData);
            }
          }
        }
        SendMessage(hMainWnd, AKD_FREETEXT, 0, (LPARAM)wszSelection);
      }
      EndDialog(hDlg, 0);
      return TRUE;
    }
    else if (LOWORD(wParam) == IDCANCEL)
    {
      EndDialog(hDlg, 0);
      return TRUE;
    }
  }
  else if (uMsg == WM_CLOSE)
  {
    PostMessage(hDlg, WM_COMMAND, IDCANCEL, 0);
    return TRUE;
  }
  return FALSE;
}

BOOL SetupCryptoClient()
{
  // Ensure that the default cryptographic client is set up.
  HCRYPTPROV hProv=0;
  HCRYPTKEY hKey=0;

  // Attempt to acquire a handle to the default key container.
  if (!CryptAcquireContextA(&hProv, "Crypt plugin", MS_DEF_PROV, PROV_RSA_FULL, 0))
  {
    // Some sort of error occured, create default key container.
    if (!CryptAcquireContextA(&hProv, "Crypt plugin", MS_DEF_PROV, PROV_RSA_FULL, CRYPT_NEWKEYSET))
    {
      // Error creating key container!
      return FALSE;
    }
  }

  // Attempt to get handle to signature key.
  if (!CryptGetUserKey(hProv, AT_SIGNATURE, &hKey))
  {
    if ((int)GetLastError() == NTE_NO_KEY)
    {
      // Create signature key pair.
      if (!CryptGenKey(hProv, AT_SIGNATURE, 0, &hKey))
      {
        // Error during CryptGenKey!
        CryptReleaseContext(hProv, 0);
        return FALSE;
      }
      else
      {
        CryptDestroyKey(hKey);
      }
    }
    else
    {
      // Error during CryptGetUserKey!
      CryptReleaseContext(hProv, 0);
      return FALSE;
    }
  }

  // Attempt to get handle to exchange key.
  if (!CryptGetUserKey(hProv, AT_KEYEXCHANGE, &hKey))
  {
    if ((int)GetLastError() == NTE_NO_KEY)
    {
      // Create key exchange key pair.
      if (!CryptGenKey(hProv, AT_KEYEXCHANGE, 0, &hKey))
      {
        // Error during CryptGenKey!
        CryptReleaseContext(hProv, 0);
        return FALSE;
      }
      else
      {
        CryptDestroyKey(hKey);
      }
    }
    else
    {
      // Error during CryptGetUserKey!
      CryptReleaseContext(hProv, 0);
      return FALSE;
    }
  }

  CryptReleaseContext(hProv, 0);
  return TRUE;
}

BOOL EncryptString(unsigned char *lpData, DWORD *dwDataLen, unsigned char *lpKey, DWORD dwKeyLen)
{
  HCRYPTPROV hProv=0;
  HCRYPTKEY hKey=0;
  HCRYPTHASH hHash=0;
  BOOL bResult=FALSE;

  // Get handle to default provider.
  if (CryptAcquireContextA(&hProv, "Crypt plugin", MS_DEF_PROV, PROV_RSA_FULL, 0))
  {
    // Create hash object.
    if (CryptCreateHash(hProv, CALG_MD5, 0, 0, &hHash))
    {
      // Hash password string.
      if (CryptHashData(hHash, lpKey, dwKeyLen, 0))
      {
        // Create block cipher session key based on hash of the password.
        if (CryptDeriveKey(hProv, ENCRYPT_ALGORITHM, hHash, CRYPT_EXPORTABLE, &hKey))
        {
          // Encrypt data
          if (CryptEncrypt(hKey, 0, TRUE, 0, lpData, dwDataLen, *dwDataLen))
          {
            bResult=TRUE;
          }
          CryptDestroyKey(hKey);
        }
      }
      CryptDestroyHash(hHash);
    }
    CryptReleaseContext(hProv, 0);
  }
  return bResult;
}

BOOL DecryptString(unsigned char *lpData, DWORD *dwDataLen, unsigned char *lpKey, DWORD dwKeyLen)
{
  HCRYPTPROV hProv=0;
  HCRYPTKEY hKey=0;
  HCRYPTHASH hHash=0;
  BOOL bResult=FALSE;

  // Get handle to default provider.
  if (CryptAcquireContextA(&hProv, "Crypt plugin", MS_DEF_PROV, PROV_RSA_FULL, 0))
  {
    // Create hash object.
    if (CryptCreateHash(hProv, CALG_MD5, 0, 0, &hHash))
    {
      // Hash password string.
      if (CryptHashData(hHash, lpKey, dwKeyLen, 0))
      {
        // Create block cipher session key based on hash of the password.
        if (CryptDeriveKey(hProv, ENCRYPT_ALGORITHM, hHash, CRYPT_EXPORTABLE, &hKey))
        {
          if (CryptDecrypt(hKey, 0, TRUE, 0, lpData, dwDataLen))
          {
            bResult=TRUE;
          }
          CryptDestroyKey(hKey);
        }
      }
      CryptDestroyHash(hHash);
    }
    CryptReleaseContext(hProv, 0);
  }
  return bResult;
}

char* GetLangStringA(LANGID wLangID, int nStringID)
{
  static char szStringBuf[MAX_PATH];

  WideCharToMultiByte(CP_ACP, 0, GetLangStringW(wLangID, nStringID), -1, szStringBuf, MAX_PATH, NULL, NULL);
  return szStringBuf;
}

wchar_t* GetLangStringW(LANGID wLangID, int nStringID)
{
  if (wLangID == LANG_RUSSIAN)
  {
    if (nStringID == STRID_ENCRYPTION)
      return L"\x0428\x0438\x0444\x0440\x043E\x0432\x0430\x043D\x0438\x0435";
    if (nStringID == STRID_DECRYPTION)
      return L"\x0420\x0430\x0441\x0448\x0438\x0444\x0440\x043E\x0432\x044B\x0432\x0430\x043D\x0438\x0435";
    if (nStringID == STRID_PASS1)
      return L"\x041F\x0430\x0440\x043E\x043B\x044C\x003A";
    if (nStringID == STRID_PASS2)
      return L"\x041F\x0430\x0440\x043E\x043B\x044C\x0020\x0435\x0449\x0435\x0020\x0440\x0430\x0437\x003A";
    if (nStringID == STRID_PASSDONTMATCH)
      return L"\x041F\x0430\x0440\x043E\x043B\x0438\x0020\x043D\x0435\x0020\x0441\x043E\x0432\x043F\x0430\x0434\x0430\x044E\x0442\x002E";
    if (nStringID == STRID_NOSELECTION)
      return L"\x041E\x0442\x0441\x0443\x0442\x0441\x0442\x0432\x0443\x0435\x0442\x0020\x0432\x044B\x0434\x0435\x043B\x0435\x043D\x0438\x0435\x0020\x0442\x0435\x043A\x0441\x0442\x0430\x002E";
    if (nStringID == STRID_OK)
      return L"\x004F\x004B";
    if (nStringID == STRID_CANCEL)
      return L"\x041E\x0442\x043C\x0435\x043D\x0430";
  }
  else
  {
    if (nStringID == STRID_ENCRYPTION)
      return L"Encryption";
    if (nStringID == STRID_DECRYPTION)
      return L"Decryption";
    if (nStringID == STRID_PASS1)
      return L"Password:";
    if (nStringID == STRID_PASS2)
      return L"Retype password:";
    if (nStringID == STRID_PASSDONTMATCH)
      return L"Passwords do not match.";
    if (nStringID == STRID_NOSELECTION)
      return L"No text selected.";
    if (nStringID == STRID_OK)
      return L"OK";
    if (nStringID == STRID_CANCEL)
      return L"Cancel";
  }
  return L"";
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hInstanceDLL=pd->hInstanceDLL;
  hMainWnd=pd->hMainWnd;
  hWndEdit=pd->hWndEdit;
  hMainIcon=pd->hMainIcon;
  bOldWindows=pd->bOldWindows;
  bAkelEdit=pd->bAkelEdit;
  wLangSystem=PRIMARYLANGID(pd->wLangSystem);
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
  }
  return TRUE;
}
